@@ -283,6 +283,8 @@ meta_monitor_config_finalize (GObject *object)
   MetaMonitorConfig *self = META_MONITOR_CONFIG (object);
 
   g_hash_table_destroy (self->configs);
+
+  G_OBJECT_CLASS (meta_monitor_config_parent_class)->finalize (object);
 }
 
 static void
@@ -462,7 +464,7 @@ handle_end_element (GMarkupParseContext  *context,
       {
         if (strcmp (element_name, "configuration") == 0 && parser->unknown_count == 0)
           {
-            MetaConfiguration *config = g_slice_new (MetaConfiguration);
+            MetaConfiguration *config = config_new ();
 
             g_assert (parser->key_array->len == parser->output_array->len);
 
@@ -923,23 +925,13 @@ key_is_laptop (MetaOutputKey *key)
 {
   /* FIXME: extend with better heuristics */
   return g_str_has_prefix (key->connector, "LVDS") ||
+    g_str_has_prefix (key->connector, "lvds") ||
+    g_str_has_prefix (key->connector, "Lvds") ||
+    g_str_has_prefix (key->connector, "LCD")  || /* some versions of fglrx, sigh */
+    g_str_has_prefix (key->connector, "DSI") ||
     g_str_has_prefix (key->connector, "eDP");
 }
 
-static gboolean
-output_is_laptop (MetaOutput *output)
-{
-  /* FIXME: extend with better heuristics */
-  switch (output->connector_type)
-    {
-    case META_CONNECTOR_TYPE_eDP:
-    case META_CONNECTOR_TYPE_LVDS:
-      return TRUE;
-    default:
-      return FALSE;
-    }
-}
-
 static gboolean
 laptop_display_is_on (MetaConfiguration *config)
 {
@@ -1061,6 +1053,17 @@ apply_configuration_with_lid (MetaMonitorConfig  *self,
     return apply_configuration (self, config, manager);
 }
 
+gboolean
+meta_monitor_config_get_is_builtin_display_on (MetaMonitorConfig *self)
+{
+  g_return_val_if_fail (META_IS_MONITOR_CONFIG (self), FALSE);
+
+  if (self->current)
+    return laptop_display_is_on (self->current);
+
+  return FALSE;
+}
+
 gboolean
 meta_monitor_config_apply_stored (MetaMonitorConfig  *self,
 				  MetaMonitorManager *manager)
@@ -1102,7 +1105,7 @@ find_primary_output (MetaOutput *outputs,
 
   for (i = 0; i < n_outputs; i++)
     {
-      if (output_is_laptop (&outputs[i]))
+      if (meta_output_is_laptop (&outputs[i]))
         return i;
     }
 
@@ -1564,18 +1567,19 @@ meta_monitor_config_restore_previous (MetaMonitorConfig  *self,
       /* The user chose to restore the previous configuration. In this
        * case, restore the previous configuration. */
       MetaConfiguration *prev_config = config_ref (self->previous);
-      apply_configuration (self, prev_config, manager);
+      gboolean ok = apply_configuration (self, prev_config, manager);
       config_unref (prev_config);
 
       /* After this, self->previous contains the rejected configuration.
        * Since it was rejected, nuke it. */
       g_clear_pointer (&self->previous, (GDestroyNotify) config_unref);
+
+      if (ok)
+        return;
     }
-  else
-    {
-      if (!meta_monitor_config_apply_stored (self, manager))
-        meta_monitor_config_make_default (self, manager);
-    }
+
+  if (!meta_monitor_config_apply_stored (self, manager))
+    meta_monitor_config_make_default (self, manager);
 }
 
 #ifdef HAVE_UPOWER_GLIB
@@ -1809,19 +1813,6 @@ crtc_can_drive_output (MetaCRTC   *crtc,
   return FALSE;
 }
 
-static gboolean
-output_supports_mode (MetaOutput      *output,
-                      MetaMonitorMode *mode)
-{
-  unsigned int i;
-
-  for (i = 0; i < output->n_modes; i++)
-    if (output->modes[i] == mode)
-      return TRUE;
-
-  return FALSE;
-}
-
 static gboolean
 crtc_assignment_assign (CrtcAssignment       *assign,
 			MetaCRTC             *crtc,
@@ -1836,9 +1827,6 @@ crtc_assignment_assign (CrtcAssignment       *assign,
   if (!crtc_can_drive_output (crtc, output))
     return FALSE;
 
-  if (!output_supports_mode (output, mode))
-    return FALSE;
-
   if ((crtc->all_transforms & (1 << transform)) == 0)
     return FALSE;
 
@@ -1925,10 +1913,9 @@ static gboolean
 real_assign_crtcs (CrtcAssignment     *assignment,
                    unsigned int        output_num)
 {
-  MetaMonitorMode *modes;
   MetaCRTC *crtcs;
   MetaOutput *outputs;
-  unsigned int n_crtcs, n_modes, n_outputs;
+  unsigned int n_crtcs, n_outputs;
   MetaOutputKey *output_key;
   MetaOutputConfig *output_config;
   unsigned int i;
@@ -1944,7 +1931,7 @@ real_assign_crtcs (CrtcAssignment     *assignment,
     return real_assign_crtcs (assignment, output_num + 1);
 
   meta_monitor_manager_get_resources (assignment->manager,
-                                      &modes, &n_modes,
+                                      NULL, NULL,
                                       &crtcs, &n_crtcs,
                                       &outputs, &n_outputs);
 
@@ -1961,9 +1948,9 @@ real_assign_crtcs (CrtcAssignment     *assignment,
           MetaOutput *output = find_output_by_key (outputs, n_outputs, output_key);
           unsigned int j;
 
-          for (j = 0; j < n_modes; j++)
+          for (j = 0; j < output->n_modes; j++)
 	    {
-              MetaMonitorMode *mode = &modes[j];
+              MetaMonitorMode *mode = output->modes[j];
               int width, height;
 
               if (meta_monitor_transform_is_rotated (output_config->transform))
@@ -1988,7 +1975,7 @@ real_assign_crtcs (CrtcAssignment     *assignment,
                                 output_config->transform,
                                 pass);
 
-                  if (crtc_assignment_assign (assignment, crtc, &modes[j],
+                  if (crtc_assignment_assign (assignment, crtc, mode,
                                               output_config->rect.x, output_config->rect.y,
                                               output_config->transform,
                                               output))
@@ -2041,7 +2028,11 @@ meta_monitor_config_assign_crtcs (MetaConfiguration  *config,
 
   all_outputs = meta_monitor_manager_get_outputs (manager,
                                                   &n_outputs);
-  g_assert (n_outputs == config->n_outputs);
+  if (n_outputs != config->n_outputs)
+    {
+      g_hash_table_destroy (assignment.info);
+      return FALSE;
+    }
 
   for (i = 0; i < n_outputs; i++)
     {
