@@ -333,12 +333,6 @@ meta_button_function_get_type (void)
         { META_BUTTON_FUNCTION_MINIMIZE, "META_BUTTON_FUNCTION_MINIMIZE", "minimize" },
         { META_BUTTON_FUNCTION_MAXIMIZE, "META_BUTTON_FUNCTION_MAXIMIZE", "maximize" },
         { META_BUTTON_FUNCTION_CLOSE, "META_BUTTON_FUNCTION_CLOSE", "close" },
-        { META_BUTTON_FUNCTION_SHADE, "META_BUTTON_FUNCTION_SHADE", "shade" },
-        { META_BUTTON_FUNCTION_ABOVE, "META_BUTTON_FUNCTION_ABOVE", "above" },
-        { META_BUTTON_FUNCTION_STICK, "META_BUTTON_FUNCTION_STICK", "stick" },
-        { META_BUTTON_FUNCTION_UNSHADE, "META_BUTTON_FUNCTION_UNSHADE", "unshade" },
-        { META_BUTTON_FUNCTION_UNABOVE, "META_BUTTON_FUNCTION_UNABOVE", "unabove" },
-        { META_BUTTON_FUNCTION_UNSTICK, "META_BUTTON_FUNCTION_UNSTICK", "unstick" },
         { META_BUTTON_FUNCTION_APPMENU, "META_BUTTON_FUNCTION_APPMENU", "appmenu" },
         { META_BUTTON_FUNCTION_LAST, "META_BUTTON_FUNCTION_LAST", "last" },
         { 0, NULL, NULL }
@@ -388,8 +382,6 @@ meta_stack_layer_get_type (void)
         { META_LAYER_NORMAL, "META_LAYER_NORMAL", "normal" },
         { META_LAYER_TOP, "META_LAYER_TOP", "top" },
         { META_LAYER_DOCK, "META_LAYER_DOCK", "dock" },
-        { META_LAYER_FULLSCREEN, "META_LAYER_FULLSCREEN", "fullscreen" },
-        { META_LAYER_FOCUSED_WINDOW, "META_LAYER_FOCUSED_WINDOW", "focused-window" },
         { META_LAYER_OVERRIDE_REDIRECT, "META_LAYER_OVERRIDE_REDIRECT", "override-redirect" },
         { META_LAYER_LAST, "META_LAYER_LAST", "last" },
         { 0, NULL, NULL }
@@ -507,6 +499,29 @@ meta_tab_show_type_get_type (void)
 
   return g_enum_type_id__volatile;
 }
+GType
+meta_pad_action_type_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { META_PAD_ACTION_BUTTON, "META_PAD_ACTION_BUTTON", "button" },
+        { META_PAD_ACTION_RING, "META_PAD_ACTION_RING", "ring" },
+        { META_PAD_ACTION_STRIP, "META_PAD_ACTION_STRIP", "strip" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("MetaPadActionType"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
 
 /* enumerations from "meta/main.h" */
 #include "meta/main.h"
